from PySide2 import QtWidgets, QtGui, QtCore
import logging

# from global_widgets.global_typeval_popup import TypeValuePopup


class SignallingLineEditWidget(QtWidgets.QLineEdit):
    manualChanged = QtCore.Signal()

    def __init__(self, NativeUI, popup, label):
        super().__init__()
        self.installEventFilter(self)
        self.label_text = label
        self.NativeUI = NativeUI
        self.popUp = popup  # NativeUI.typeValPopupAlpha
        # self.popUp = TypeValuePopup(NativeUI)#,'text edit',0,1,2,3,4)
        # self.popUp.lineEdit.setValidator(None)  # nsure it accepts text
        # self.popUp.okButton.clicked.connect(self.okButtonPressed)
        # self.popUp.cancelButton.clicked.connect(self.cancelButtonPressed)

    def okButtonPressed(self):
        val = self.popUp.lineEdit.text()
        self.setText(val)
        self.popUp.close()
        self.manualChanged.emit()

    def cancelButtonPressed(self):
        self.popUp.lineEdit.setText(self.popUp.lineEditp.saveVal)
        self.popUp.close()

    def eventFilter(self, source, event):
        if source is self and event.type() == QtCore.QEvent.MouseButtonDblClick:
            self.popUp.populatePopup(self, self.NativeUI.display_stack.currentWidget())
            self.NativeUI.display_stack.setCurrentWidget(self.popUp)
            return True
        return False

    def value(self):
        return self.text()

    def setValue(self, value):
        self.setText(str(value))


class LabelledLineEditWidget(QtWidgets.QWidget):
    def __init__(self, NativeUI, popup, infoArray, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # print(infoArray)
        self.NativeUI = NativeUI
        self.cmd_type, self.cmd_code = "", ""
        self.min, self.max, self.step = 0, 10000, 0.3
        self.decPlaces = 2
        if len(infoArray) == 9:
            self.label, self.units, self.tag, self.cmd_type, self.cmd_code, self.min, self.max, self.step, self.decPlaces = (
                infoArray
            )
        elif len(infoArray) == 6:
            self.label, self.units, self.tag, self.cmd_type, self.cmd_code, self.initText = (
                infoArray
            )
        elif len(infoArray) == 3:
            self.label, self.units, self.tag = infoArray
        self.manuallyUpdated = False
        layout = QtWidgets.QHBoxLayout()
        self.widgetList = []

        # if self.label != "":
        self.nameLabel = QtWidgets.QLabel()
        self.nameLabel.setFont(NativeUI.text_font)
        self.nameLabel.setStyleSheet(
            "color:" + NativeUI.colors["page_foreground"].name() + ";"
        )
        self.nameLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.widgetList.append(self.nameLabel)

        self.simpleSpin = SignallingLineEditWidget(NativeUI, popup, self.label)
        self.simpleSpin.setText(self.initText)
        self.simpleSpin.setStyleSheet(
            "QLineEdit{"
            "    color:" + NativeUI.colors["page_foreground"].name() + ";"
            "}"
            "QLineEdit[bgColour='1']{"
            "    background-color:" + NativeUI.colors["page_foreground"].name() + ";"
            "}"
            "QLineEdit[bgColour='0']{"
            "    background-color:" + NativeUI.colors["page_background"].name() + ";"
            "}"
            "QLineEdit[textColour='1']{"
            "    color:" + NativeUI.colors["page_background"].name() + ";"
            "}"
            "QLineEdit[textColour='0']{"
            "    color:" + NativeUI.colors["page_foreground"].name() + ";"
            "}"
            "QLineEdit[textColour='2']{"
            "    color:" + NativeUI.colors["red"].name() + ";"
            "}"
        )
        self.simpleSpin.setFixedWidth(600)
        self.simpleSpin.setFont(NativeUI.text_font)
        self.simpleSpin.setProperty("textColour", "1")
        self.simpleSpin.setProperty("bgColour", "1")
        self.simpleSpin.setAlignment(QtCore.Qt.AlignCenter)
        self.simpleSpin.textChanged.connect(self.textUpdate)
        if self.cmd_type == "":
            self.simpleSpin.setReadOnly(True)
            self.simpleSpin.setProperty("bgColour", "1")
        self.widgetList.append(self.simpleSpin)

        # if self.units != "":
        self.unitLabel = QtWidgets.QLabel()
        self.unitLabel.setFont(NativeUI.text_font)
        self.unitLabel.setStyleSheet(
            "color:" + NativeUI.colors["page_foreground"].name() + ""
        )
        self.unitLabel.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.widgetList.append(self.unitLabel)

        for widget in self.widgetList:
            layout.addWidget(widget)

        self.setLayout(layout)
        self.localise_text(NativeUI.text)

    def textUpdate(self):
        self.manuallyUpdated = True
        self.simpleSpin.setProperty("textColour", "2")
        self.simpleSpin.style().polish(self.simpleSpin)

    def update_value(self, db):
        newVal = db[self.tag]
        if newVal == {}:
            a = 1  # do nothing
        else:
            logging.debug("got a personal db")
            self.simpleSpin.setText(newVal)
            self.simpleSpin.setProperty("textColour", "1")
            self.simpleSpin.style().polish(self.simpleSpin)

    def get_value(self):
        return self.simpleSpin.text()

    def set_size(self, x: int, y: int, spacing: int = 5) -> int:
        """
        Set the size of the widget.
        Also rescale the elements within it to equally distribute the width
        """

        if x is not None:
            self.setFixedWidth(x)
            x_widget = int(x / 8)
            self.nameLabel.setFixedWidth(3 * x_widget - spacing)
            self.simpleSpin.setFixedWidth(3 * x_widget - spacing)
            self.unitLabel.setFixedWidth(2 * x_widget - spacing)
        if y is not None:
            self.setFixedHeight(y)
            for widget in self.widgetList:
                widget.setFixedHeight(y)
        # self.layout.setSpacing(spacing)
        return 0

    def localise_text(self, text: dict) -> int:
        """
        Source the text for the labels from the specified dictionary.
        """
        if self.label == "":
            self.nameLabel.setText("")
        else:
            self.nameLabel.setText(text[self.tag])

        if self.units == "":
            self.unitLabel.setText("")
        else:
            self.unitLabel.setText(text[self.units])

        return 0
